/*
 * Notification support.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <mutex>
#include <set>
#include <vector>

#include <systypes/base.h>



/* Collection data types. */

typedef std::set<l4_cap_idx_t> NotifierSet;

/* Notification endpoint details. */

class NotificationEndpoint
{
public:
    NotifierSet notifiers;
    notify_flags_t flags = 0, deferred = 0;
};

typedef std::vector<NotificationEndpoint> NotificationEndpointList;



/* Notification support, allowing pipe or file users to receive events due to
   activity on those objects. */

class NotificationSupport
{
protected:
    std::mutex _lock;

    /* Endpoint count. */

    unsigned int _min_endpoints;

    /* Notification endpoint details. */

    NotificationEndpointList _endpoints;

    /* Deferred notifications for new endpoints. */

    notify_flags_t _deferred = 0;

    virtual void release_notifiers();

    virtual void _notify(unsigned int endpoint, notify_flags_t flags);

    virtual void _subscribe(unsigned int endpoint, l4_cap_idx_t notifier, notify_flags_t flags);

public:
    explicit NotificationSupport(unsigned int endpoints=0);

    virtual ~NotificationSupport();

    virtual void notify(unsigned int endpoint, notify_flags_t flags);

    virtual void notify_all(notify_flags_t flags);

    virtual void notify_others(unsigned int endpoint, notify_flags_t flags);

    virtual unsigned int subscribe(l4_cap_idx_t notifier, notify_flags_t flags);

    virtual void subscribe(unsigned int endpoint, l4_cap_idx_t notifier, notify_flags_t flags);
};

// vim: tabstop=4 expandtab shiftwidth=4
